diff -r c02918434721 src/lib-storage/index/imapc/imapc-list.c
--- a/src/lib-storage/index/imapc/imapc-list.c	Thu Oct 02 12:31:34 2014 +0300
+++ b/src/lib-storage/index/imapc/imapc-list.c	Fri Mar 20 20:26:21 2015 +0200
@@ -248,7 +248,7 @@
 
 static void imapc_list_sep_verify(struct imapc_mailbox_list *list)
 {
-	const char *imapc_list_prefix = list->set->imapc_list_prefix;
+	const char *imapc_list_prefix = list->set->list_prefix;
 
 	if (list->root_sep == '\0') {
 		mailbox_list_set_critical(&list->list,
@@ -320,7 +320,7 @@
 imapc_list_get_storage_name(struct mailbox_list *_list, const char *vname)
 {
 	struct imapc_mailbox_list *list = (struct imapc_mailbox_list *)_list;
-	const char *prefix = list->set->imapc_list_prefix;
+	const char *prefix = list->set->list_prefix;
 	const char *storage_name;
 
 	storage_name = mailbox_list_default_get_storage_name(_list, vname);
@@ -337,7 +337,7 @@
 imapc_list_get_vname(struct mailbox_list *_list, const char *storage_name)
 {
 	struct imapc_mailbox_list *list = (struct imapc_mailbox_list *)_list;
-	const char *prefix = list->set->imapc_list_prefix;
+	const char *prefix = list->set->list_prefix;
 	unsigned int prefix_len;
 
 	if (*storage_name == '\0') {
@@ -400,7 +400,7 @@
 		return NULL;
 
 	vname = mailbox_list_get_vname(&list->list, name);
-	if (list->set->imapc_list_prefix[0] != '\0') {
+	if (list->set->list_prefix[0] != '\0') {
 		/* put back the prefix, so it gets included in the filesystem. */
 		unsigned int vname_len = strlen(vname);
 
@@ -416,10 +416,10 @@
 		}
 		if (vname[0] == '\0') {
 			vname = t_strconcat(ns->prefix,
-				list->set->imapc_list_prefix, NULL);
+				list->set->list_prefix, NULL);
 		} else {
 			vname = t_strdup_printf("%s%s%c%s", ns->prefix,
-						list->set->imapc_list_prefix,
+						list->set->list_prefix,
 						ns_sep, vname);
 		}
 	}
@@ -481,8 +481,8 @@
 	struct mailbox_list *fs_list = imapc_list_get_fs(list);
 	struct mailbox_list_iterate_context *iter;
 	const struct mailbox_info *info;
-	const char *imapc_list_prefix = list->set->imapc_list_prefix;
-	unsigned int imapc_list_prefix_len = strlen(imapc_list_prefix);
+	const char *imapc_list_prefix = list->set->list_prefix;
+	unsigned int list_prefix_len = strlen(imapc_list_prefix);
 	const char *fs_name, *vname;
 
 	if (fs_list == NULL)
@@ -494,7 +494,7 @@
 				      MAILBOX_LIST_ITER_RETURN_NO_FLAGS);
 	while ((info = mailbox_list_iter_next(iter)) != NULL) T_BEGIN {
 		vname = info->vname;
-		if (imapc_list_prefix_len > 0 &&
+		if (list_prefix_len > 0 &&
 		    strcasecmp(vname, "INBOX") != 0) {
 			/* skip over the namespace prefix */
 			i_assert(strncmp(vname, fs_list->ns->prefix,
@@ -502,8 +502,8 @@
 			vname += fs_list->ns->prefix_len;
 			/* skip over the imapc list prefix */
 			i_assert(strncmp(vname, imapc_list_prefix,
-					 imapc_list_prefix_len) == 0);
-			vname += imapc_list_prefix_len;
+					 list_prefix_len) == 0);
+			vname += list_prefix_len;
 			if (vname[0] != '\0') {
 				i_assert(vname[0] == mail_namespace_get_sep(fs_list->ns));
 				vname++;
@@ -533,13 +533,13 @@
 	if (list->refreshed_mailboxes)
 		return 0;
 
-	if (*list->set->imapc_list_prefix == '\0')
+	if (*list->set->list_prefix == '\0')
 		pattern = "*";
 	else {
 		/* list "prefix*" instead of "prefix.*". this may return a bit
 		   more than we want, but we're also interested in the flags
 		   of the prefix itself. */
-		pattern = t_strdup_printf("%s*", list->set->imapc_list_prefix);
+		pattern = t_strdup_printf("%s*", list->set->list_prefix);
 	}
 
 	cmd = imapc_list_simple_context_init(&ctx, list);
@@ -555,7 +555,7 @@
 		   imapc_list_prefix doesn't match it */
 		bool created;
 		node = mailbox_tree_get(list->mailboxes, "INBOX", &created);
-		if (*list->set->imapc_list_prefix != '\0') {
+		if (*list->set->list_prefix != '\0') {
 			/* this listing didn't include the INBOX itself, but
 			   might have included its children. make sure there
 			   aren't any extra flags in it (especially
@@ -757,10 +757,10 @@
 		mailbox_tree_init(mail_namespace_get_sep(_src_list->ns));
 
 	cmd = imapc_list_simple_context_init(&ctx, src_list);
-	if (*src_list->set->imapc_list_prefix == '\0')
+	if (*src_list->set->list_prefix == '\0')
 		pattern = "*";
 	else
-		pattern = t_strdup_printf("%s*", src_list->set->imapc_list_prefix);
+		pattern = t_strdup_printf("%s*", src_list->set->list_prefix);
 	imapc_command_sendf(cmd, "LSUB \"\" %s", pattern);
 	imapc_simple_run(&ctx);
 
diff -r c02918434721 src/lib-storage/index/imapc/imapc-settings.c
--- a/src/lib-storage/index/imapc/imapc-settings.c	Thu Oct 02 12:31:34 2014 +0300
+++ b/src/lib-storage/index/imapc/imapc-settings.c	Fri Mar 20 20:26:21 2015 +0200
@@ -7,49 +7,78 @@
 
 #include <stddef.h>
 
+/* we're kind of kludging here to avoid "imapc_" prefix in the struct fields */
+#undef DEF
+#define DEF(type, name) \
+	{ type, "imapc_"#name, offsetof(struct imapc_settings, name), NULL }
+
+static bool imapc_settings_check(void *_set, pool_t pool, const char **error_r);
+
+static const struct setting_define imapc_setting_defines[] = {
+	/* NOTE: update also imapc_ingroup_setting_defines[] */
+	DEF(SET_STR, host),
+	DEF(SET_UINT, port),
+
+	DEF(SET_STR_VARS, user),
+	DEF(SET_STR_VARS, master_user),
+	DEF(SET_STR, password),
+
+	DEF(SET_ENUM, ssl),
+	DEF(SET_BOOL, ssl_verify),
+
+	DEF(SET_STR, features),
+	DEF(SET_STR, rawlog_dir),
+	DEF(SET_STR, list_prefix),
+	DEF(SET_TIME, max_idle_time),
+
+	SETTING_DEFINE_LIST_END
+};
+
 #undef DEF
 #define DEF(type, name) \
 	{ type, #name, offsetof(struct imapc_settings, name), NULL }
 
-static bool imapc_settings_check(void *_set, pool_t pool, const char **error_r);
+static const struct setting_define imapc_ingroup_setting_defines[] = {
+	/* NOTE: update also imapc_setting_defines[] */
+	DEF(SET_STR, name),
+	DEF(SET_STR, host),
+	DEF(SET_UINT, port),
 
-static const struct setting_define imapc_setting_defines[] = {
-	DEF(SET_STR, imapc_host),
-	DEF(SET_UINT, imapc_port),
+	DEF(SET_STR_VARS, user),
+	DEF(SET_STR_VARS, master_user),
+	DEF(SET_STR, password),
 
-	DEF(SET_STR_VARS, imapc_user),
-	DEF(SET_STR_VARS, imapc_master_user),
-	DEF(SET_STR, imapc_password),
+	DEF(SET_ENUM, ssl),
+	DEF(SET_BOOL, ssl_verify),
 
-	DEF(SET_ENUM, imapc_ssl),
-	DEF(SET_BOOL, imapc_ssl_verify),
-
-	DEF(SET_STR, imapc_features),
-	DEF(SET_STR, imapc_rawlog_dir),
-	DEF(SET_STR, imapc_list_prefix),
-	DEF(SET_TIME, imapc_max_idle_time),
+	DEF(SET_STR, features),
+	DEF(SET_STR, rawlog_dir),
+	DEF(SET_STR, list_prefix),
+	DEF(SET_TIME, max_idle_time),
 
 	SETTING_DEFINE_LIST_END
 };
 
 static const struct imapc_settings imapc_default_settings = {
-	.imapc_host = "",
-	.imapc_port = 143,
+	.name = "",
 
-	.imapc_user = "",
-	.imapc_master_user = "",
-	.imapc_password = "",
+	.host = "",
+	.port = 143,
 
-	.imapc_ssl = "no:imaps:starttls",
-	.imapc_ssl_verify = TRUE,
+	.user = "",
+	.master_user = "",
+	.password = "",
 
-	.imapc_features = "",
-	.imapc_rawlog_dir = "",
-	.imapc_list_prefix = "",
-	.imapc_max_idle_time = 60*29
+	.ssl = "no:imaps:starttls",
+	.ssl_verify = TRUE,
+
+	.features = "",
+	.rawlog_dir = "",
+	.list_prefix = "",
+	.max_idle_time = 60*29
 };
 
-static const struct setting_parser_info imapc_setting_parser_info = {
+const struct setting_parser_info imapc_setting_parser_info = {
 	.module_name = "imapc",
 	.defines = imapc_setting_defines,
 	.defaults = &imapc_default_settings,
@@ -63,6 +92,46 @@
 	.check_func = imapc_settings_check
 };
 
+const struct setting_parser_info imapc_ingroup_setting_parser_info = {
+	.module_name = "imapc",
+	.defines = imapc_ingroup_setting_defines,
+	.defaults = &imapc_default_settings,
+
+	.type_offset = offsetof(struct imapc_settings, name),
+	.struct_size = sizeof(struct imapc_settings),
+
+	.parent_offset = (size_t)-1,
+	.parent = &mail_user_setting_parser_info,
+
+	.check_func = imapc_settings_check
+};
+
+#undef DEFLIST_UNIQUE
+#define DEFLIST_UNIQUE(field, name, defines) \
+	{ SET_DEFLIST_UNIQUE, name, offsetof(struct imapc_group_settings, field), defines }
+
+static const struct setting_define imapc_group_setting_defines[] = {
+	DEFLIST_UNIQUE(groups, "imapc", &imapc_ingroup_setting_parser_info),
+
+	SETTING_DEFINE_LIST_END
+};
+
+static const struct imapc_group_settings imapc_group_default_settings = {
+	.groups = ARRAY_INIT
+};
+
+const struct setting_parser_info imapc_group_setting_parser_info = {
+	.module_name = "imapc",
+	.defines = imapc_group_setting_defines,
+	.defaults = &imapc_group_default_settings,
+
+	.type_offset = (size_t)-1,
+	.struct_size = sizeof(struct imapc_group_settings),
+
+	.parent_offset = (size_t)-1,
+	.parent = &mail_user_setting_parser_info
+};
+
 const struct setting_parser_info *imapc_get_setting_parser_info(void)
 {
 	return &imapc_setting_parser_info;
@@ -89,7 +158,7 @@
         const struct imapc_feature_list *list;
 	const char *const *str;
 
-        str = t_strsplit_spaces(set->imapc_features, " ,");
+        str = t_strsplit_spaces(set->features, " ,");
 	for (; *str != NULL; str++) {
 		list = imapc_feature_list;
 		for (; list->name != NULL; list++) {
@@ -113,11 +182,11 @@
 {
 	struct imapc_settings *set = _set;
 
-	if (set->imapc_port == 0 || set->imapc_port > 65535) {
+	if (set->port == 0 || set->port > 65535) {
 		*error_r = "invalid imapc_port";
 		return FALSE;
 	}
-	if (set->imapc_max_idle_time == 0) {
+	if (set->max_idle_time == 0) {
 		*error_r = "imapc_max_idle_time must not be 0";
 		return FALSE;
 	}
diff -r c02918434721 src/lib-storage/index/imapc/imapc-settings.h
--- a/src/lib-storage/index/imapc/imapc-settings.h	Thu Oct 02 12:31:34 2014 +0300
+++ b/src/lib-storage/index/imapc/imapc-settings.h	Fri Mar 20 20:26:21 2015 +0200
@@ -10,24 +10,34 @@
 /* </settings checks> */
 
 struct imapc_settings {
-	const char *imapc_host;
-	unsigned int imapc_port;
+	const char *name;
 
-	const char *imapc_user;
-	const char *imapc_master_user;
-	const char *imapc_password;
+	const char *host;
+	unsigned int port;
 
-	const char *imapc_ssl;
-	bool imapc_ssl_verify;
+	const char *user;
+	const char *master_user;
+	const char *password;
 
-	const char *imapc_features;
-	const char *imapc_rawlog_dir;
-	const char *imapc_list_prefix;
-	unsigned int imapc_max_idle_time;
+	const char *ssl;
+	bool ssl_verify;
+
+	const char *features;
+	const char *rawlog_dir;
+	const char *list_prefix;
+	unsigned int max_idle_time;
 
 	enum imapc_features parsed_features;
 };
 
+struct imapc_group_settings {
+	ARRAY(struct imapc_settings *) groups;
+};
+
+extern const struct setting_parser_info imapc_setting_parser_info;
+extern const struct setting_parser_info imapc_ingroup_setting_parser_info;
+extern const struct setting_parser_info imapc_group_setting_parser_info;
+
 const struct setting_parser_info *imapc_get_setting_parser_info(void);
 
 #endif
diff -r c02918434721 src/lib-storage/index/imapc/imapc-storage.c
--- a/src/lib-storage/index/imapc/imapc-storage.c	Thu Oct 02 12:31:34 2014 +0300
+++ b/src/lib-storage/index/imapc/imapc-storage.c	Fri Mar 20 20:26:21 2015 +0200
@@ -203,31 +203,31 @@
 	string_t *str;
 
 	memset(&set, 0, sizeof(set));
-	set.host = imapc_set->imapc_host;
+	set.host = imapc_set->host;
 	if (*set.host == '\0') {
-		*error_r = "missing imapc_host";
+		*error_r = "missing host";
 		return -1;
 	}
-	set.port = imapc_set->imapc_port;
-	if (imapc_set->imapc_user[0] != '\0')
-		set.username = imapc_set->imapc_user;
+	set.port = imapc_set->port;
+	if (imapc_set->user[0] != '\0')
+		set.username = imapc_set->user;
 	else if (ns->owner != NULL)
 		set.username = ns->owner->username;
 	else
 		set.username = ns->user->username;
-	set.master_user = imapc_set->imapc_master_user;
-	set.password = imapc_set->imapc_password;
+	set.master_user = imapc_set->master_user;
+	set.password = imapc_set->password;
 	if (*set.password == '\0') {
-		*error_r = "missing imapc_password";
+		*error_r = "missing password";
 		return -1;
 	}
-	set.max_idle_time = imapc_set->imapc_max_idle_time;
+	set.max_idle_time = imapc_set->max_idle_time;
 	set.dns_client_socket_path = *ns->user->set->base_dir == '\0' ? "" :
 		t_strconcat(ns->user->set->base_dir, "/",
 			    DNS_CLIENT_SOCKET_NAME, NULL);
 	set.debug = mail_set->mail_debug;
 	set.rawlog_dir = mail_user_home_expand(ns->user,
-					       imapc_set->imapc_rawlog_dir);
+					       imapc_set->rawlog_dir);
 
 	str = t_str_new(128);
 	mail_user_set_get_temp_prefix(str, ns->user->set);
@@ -235,10 +235,10 @@
 
 	set.ssl_ca_dir = mail_set->ssl_client_ca_dir;
 	set.ssl_ca_file = mail_set->ssl_client_ca_file;
-	set.ssl_verify = imapc_set->imapc_ssl_verify;
-	if (strcmp(imapc_set->imapc_ssl, "imaps") == 0)
+	set.ssl_verify = imapc_set->ssl_verify;
+	if (strcmp(imapc_set->ssl, "imaps") == 0)
 		set.ssl_mode = IMAPC_CLIENT_SSL_MODE_IMMEDIATE;
-	else if (strcmp(imapc_set->imapc_ssl, "starttls") == 0)
+	else if (strcmp(imapc_set->ssl, "starttls") == 0)
 		set.ssl_mode = IMAPC_CLIENT_SSL_MODE_STARTTLS;
 	else
 		set.ssl_mode = IMAPC_CLIENT_SSL_MODE_NONE;
